
/* * 
 * This file is part of bufferpool
 *
 * Copyright (C) 2007 by LScube team <team@streaming.polito.it>
 * See AUTHORS for more details
 * 
 * bufferpool is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * bufferpool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with bufferpool; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 *
 */
#include <bufferpool.h>

/*! return the next packet timestamp
 *
 * @return -1 on error, the packet timestamp otherwise
 **/

double bp_nextts(BPConsumer * cons)
{
    BPSlot *last_read;
    BPSlot *next;
    int ret = -1;

    bp_lock(cons->buffer);

    if(bp_shm_refresh(cons->buffer))
        goto err_unlock;

    last_read = BPtoSlot(cons->buffer, cons->last_read_pos);
    next = &cons->buffer->slots[cons->read_pos];

    if (!next->refs || (next->slot_seq < cons->last_seq)) {
        // added some slots?
        if (last_read && cons->buffer->slots[last_read->next].refs
            && (cons->buffer->slots[last_read->next].slot_seq >
            cons->last_seq))
            next = &cons->buffer->slots[last_read->next];
        else {
            goto err_unlock;
        }
    } else if (last_read
           && (cons->buffer->slots[last_read->next].slot_seq <
               next->slot_seq))
        next = &cons->buffer->slots[last_read->next];
    
    ret = next->timestamp;

    err_unlock:
    bp_unlock(cons->buffer);

    return ret;
}
